// Applet : GeneralClient
// Author : Russ Ethington
// Version: 1/28/96 for Java 1.0

import java.lang.*; 
import java.net.*;
import java.io.*;

class GeneralClient implements Runnable
{
   private ConnectionHandler owner;
   private Socket            socket;

   private String            host;
   private int               port;
   
   private Thread            life = null;
   private DataInputStream   in = null;
   private PrintStream       out = null;
   
   private boolean           runState = false;
   private boolean           connectState = false;
   final int                 defaultPort = 4646;

   public GeneralClient(ConnectionHandler theOwner)
   {
      owner = theOwner;
   }

   public void start(String theHost) 
   {
      start(theHost, defaultPort);
   }

   public void start(String theHost, int thePort)
   {
      owner.debugLog("GeneralClient:start", this);
      runState = true;
      host = theHost;
      port = thePort;

      if (life == null)
      {
         life = new Thread(this);
         life.setPriority(Thread.MAX_PRIORITY);
      }
      life.start();      
   }

   public void stop()
   {
      owner.debugLog("GeneralClient:stop", this);
      if (life != null) 
         life.stop();
      shutdown();
   }

   public boolean isRunning()
   {
      owner.debugLog("GeneralClient:isRunning", this);
      return runState;
   }

   public boolean isConnected()
   {
      owner.debugLog("GeneralClient:isConnected", this);
      return connectState;
   }

   public void run()
   {
      owner.debugLog("GeneralClient:run", this);
      owner.changeStatus("Connecting to '" + 
                         host + 
                         "' on port '" + 
                         port + 
                         "'...", this);
      
      try
         {socket = new Socket(host, port);}
      catch (IOException e)
      {        
         owner.connectionMishap("Could not connect to '" + host + "'.", this);
         shutdown();
         return;
      }       

      owner.connectionUp("Connected.", this);
      
      try
      {
         in = new DataInputStream(
                  new BufferedInputStream(socket.getInputStream()));
         out = new PrintStream(
                   new BufferedOutputStream(socket.getOutputStream()), true);
      }
      catch (IOException e)
      {
         owner.connectionDown("Could not create data streams.", this);
         shutdown();
         return;
      }
      connectState = true;
      owner.handleSession(in, out, this);
      connectState = false;
      owner.connectionDown("Disconnected.", this);
   }
   
   public void shutdown()
   {
      owner.debugLog("GeneralClient:shutdown", this);
      life = null;
      runState = false;
      if (connectState)
      {
         try
         {
            in.close();
            out.close();
         }  catch (IOException e) { }
         connectState = false;
         if (socket != null)
         {
            try {socket.close();}
            catch (Exception e) { }
         }
      }
      in = null;
      out = null;
      socket = null;
      life = null;
   }     
}
